<?php

namespace apexl\Io\modules\user\entities;

use apexl\Config\Singleton;
use apexl\encryption\Encrypt;
use apexl\Io\includes\Entity;
use apexl\Io\modules\user\entities\operators\refreshTokenOperator;
use apexl\Io\modules\user\entities\operators\twoFactorMethodOperator;
use apexl\Io\modules\user\entities\operators\twoFactorRecoveryCodeOperator;
use apexl\Io\modules\user\entities\operators\userNoteOperator;
use apexl\Io\operators\entityDatabaseOperator;

class twoFactorRecoveryCodeEntity extends Entity
{
    public function __construct()
    {
        parent::__construct('user_recovery_codes');
        $this->setOperator(new twoFactorRecoveryCodeOperator('user_recovery_codes', 'id'));
    }
}
