<?php

namespace apexl\Io\modules\user\entities;

use apexl\Io\includes\Entity;
use apexl\Io\modules\user\entities\operators\refreshTokenOperator;
use apexl\Io\operators\entityDatabaseOperator;

class refreshTokenEntity extends Entity
{
    public function __construct()
    {
        parent::__construct('refresh_tokens');
        $this->setOperator(new refreshTokenOperator('refresh_tokens', 'user_id'));
    }
}
